'Voronoi (11/18/2001 8:35:27 am)'.!


Form subclass: #Voronoi2dDiagram
	instanceVariableNames: 'diagramExtent diagramPoints diagramImage voronoiProcessor processingCondition releaseBlock circumscriptionCircle '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Voronoi-2d-Diagram'!

!Voronoi2dDiagram methodsFor: 'initialize-release' stamp: 'nishis 5/16/98 00:17'!
initialize
	"super initialize."
	diagramExtent := super extent.
	diagramPoints := OrderedCollection new.
	diagramImage := nil.
	voronoiProcessor := nil.
	processingCondition := true.
	circumscriptionCircle := false.
	releaseBlock := [self yourself].
	^self! !

!Voronoi2dDiagram methodsFor: 'initialize-release'!
release
	releaseBlock value! !

!Voronoi2dDiagram methodsFor: 'initialize-release'!
releaseBlock: aBlock 
	releaseBlock := aBlock! !


!Voronoi2dDiagram methodsFor: 'accessing'!
area
	| processor |
	processor := self processor.
	self condition ifFalse: [^0.0].
	^processor area! !

!Voronoi2dDiagram methodsFor: 'accessing' stamp: 'nishis 5/16/98 00:17'!
circumscriptionCircle
	^circumscriptionCircle = true! !

!Voronoi2dDiagram methodsFor: 'accessing' stamp: 'nishis 5/16/98 00:17'!
circumscriptionCircle: aBoolean
	circumscriptionCircle := aBoolean = true! !

!Voronoi2dDiagram methodsFor: 'accessing'!
condition
	^processingCondition! !

!Voronoi2dDiagram methodsFor: 'accessing' stamp: 'nishis 5/15/98 21:27'!
diagram
	diagramImage isNil ifTrue: [diagramImage := self makeImage].
	^diagramImage! !

!Voronoi2dDiagram methodsFor: 'accessing'!
extent
	^diagramExtent! !

!Voronoi2dDiagram methodsFor: 'accessing'!
extent: extentPoint 
	diagramExtent := extentPoint max: 1 @ 1.
	self flushImage! !

!Voronoi2dDiagram methodsFor: 'accessing'!
height
	^self extent y! !

!Voronoi2dDiagram methodsFor: 'accessing'!
points
	| factor |
	factor := self extent + (1 @ 1) / self extent.
	^diagramPoints collect: [:point | point scaleBy: factor]! !

!Voronoi2dDiagram methodsFor: 'accessing'!
precision
	^self processor precision! !

!Voronoi2dDiagram methodsFor: 'accessing'!
processor
	self diagram.
	^voronoiProcessor! !

!Voronoi2dDiagram methodsFor: 'accessing'!
width
	^self extent x! !


!Voronoi2dDiagram methodsFor: 'bounds accessing'!
preferredBounds
	^0 @ 0 corner: self extent! !


!Voronoi2dDiagram methodsFor: 'adding' stamp: 'nishis 5/15/98 01:15'!
add: aPoint 
	| factor point |
	self condition
		ifTrue: 
			[factor := self extent / (self extent + (1 @ 1)).
			point := aPoint x asFloat @ aPoint y asFloat scaleBy: factor.
			((self boundingBox containsPoint: point)
				and: [(diagramPoints includes: point) not])
				ifTrue: 
					[diagramPoints add: point.
					self flushImage]]! !

!Voronoi2dDiagram methodsFor: 'adding'!
addAll: points 
	points do: [:point | self add: point]! !


!Voronoi2dDiagram methodsFor: 'removing'!
remove: aPoint 
	| point |
	point := diagramPoints remove: aPoint ifAbsent: [nil].
	point isNil ifFalse: [self flushImage].
	^point! !


!Voronoi2dDiagram methodsFor: 'displaying'!
displayOn: graphicsContext 
	self asImage displayOn: graphicsContext! !


!Voronoi2dDiagram methodsFor: 'converting'!
asDelaunayDiagram
	^(Delaunay2dDiagram extent: self extent)
		addAll: self points; yourself! !

!Voronoi2dDiagram methodsFor: 'converting'!
asImage
	^self diagram! !

!Voronoi2dDiagram methodsFor: 'converting'!
asVoronoiDiagram
	^self! !


!Voronoi2dDiagram methodsFor: 'utilities'!
labelString
	^'Voronoi Diagram' copy! !

!Voronoi2dDiagram methodsFor: 'utilities' stamp: 'nishis 5/15/98 23:56'!
show
	| displayModel scheduledView displayView |
	displayModel := DisplayModel visual: self diagram.
	scheduledView := AlteredScheduledView
		model: displayModel
		label: self labelString
		minimumSize: self extent + 4
		maximumSize: self extent + 4.
	displayView := DisplayView model: displayModel.
	displayView controller: NoController new.
	scheduledView addSubView: displayView.
	scheduledView controller open! !


!Voronoi2dDiagram methodsFor: 'private'!
borderColor
	^Color green! !

!Voronoi2dDiagram methodsFor: 'private' stamp: 'nishis 5/16/98 00:17'!
circle: gc scale: factor clip: bounds 
	| processor center radius box point dots dot1 dot2 dot3 line1 line2 |
	circumscriptionCircle ifFalse: [^'do nothing'].
	processor := voronoiProcessor.
	processor startIndex to: processor voronoiPoints size
		do: 
			[:index | 
			point := processor voronoiPoints at: index.
			dots := processor dotsAtPoint: point.
			dot1 := dots at: 1.
			dot2 := dots at: 2.
			dot3 := dots at: 3.
			line1 := dot1 bisector: dot2.
			line2 := dot2 bisector: dot3.
			center := line1 intersectingPointWithLine: line2.
			center := (processor restorePoint: center)
						scaleBy: factor.
			radius := center dist: ((processor restoreDot: dot1)
							scaleBy: factor).
			(bounds containsPoint: center)
				ifTrue: 
					[box := ((center extent: 0 @ 0)
								expandBy: radius) rounded.
					voronoiProcessor
						displayCircleOn: gc 
						""radius: box height / 2 
						center: box center ""
						"box: box "
						lineWidth: 1 
						color: Color yellow]]! !

!Voronoi2dDiagram methodsFor: 'private'!
dotColor
	^Color blue! !

!Voronoi2dDiagram methodsFor: 'private'!
flushImage
	diagramImage := nil! !

!Voronoi2dDiagram methodsFor: 'private' stamp: 'nishis 5/16/98 00:37'!
makeImage
	| gc bounds factor box point1 point2 |
	voronoiProcessor := Voronoi2dProcessor dots: self points.
	processingCondition := voronoiProcessor compute.
	gc := Form extent: self extent depth: Display depth.
	bounds := gc boundingBox.
	gc fillWithColor: Color white.
	factor := self extent / (self extent + (1 @ 1)).
	voronoiProcessor 
			displayBorderOn: gc 
			box: (bounds insetBy: (0 @ 0 corner: 1 @ 1)) 
			lineWidth: 1 
			color: self borderColor.
	self
		circle: gc
		scale: factor
		clip: bounds.
	voronoiProcessor dots
		do: 
			[:dot | 
			box := (dot scaleBy: factor) rounded extent: 1 @ 1.
			box := box expandBy: (1 @ 1 corner: 1 @ 1).
			voronoiProcessor
				displayBoxOn: gc
				box: box
				clip: bounds
				lineWidth: 1 
				color: self dotColor].
	voronoiProcessor sides
		do: 
			[:side | 
			point1 := ((side at: 1)
						scaleBy: factor) rounded.
			point2 := ((side at: 2)
						scaleBy: factor) rounded.
			voronoiProcessor
				displayLineOn: gc
				from: point1
				to: point2
				clip: bounds
				lineWidth: 1 
				color: self sideColor].
	diagramImage := gc. 
	^gc! !

!Voronoi2dDiagram methodsFor: 'private'!
sideColor
	^Color red! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Voronoi2dDiagram class
	instanceVariableNames: ''!

!Voronoi2dDiagram class methodsFor: 'copyright'!
copyright
	^'Copyright 1991-1998 AOKI Atsushi'! !

!Voronoi2dDiagram class methodsFor: 'copyright'!
system
	^'Voronoi and Delaunay Diagram'! !

!Voronoi2dDiagram class methodsFor: 'copyright'!
version
	^'006'! !


!Voronoi2dDiagram class methodsFor: 'instance creation' stamp: 'nishis 5/15/98 01:23'!
extent: extentPoint 
	"^(self new) extent: extentPoint; yourself"
	^(super extent: extentPoint depth: Display depth) initialize! !

!Voronoi2dDiagram class methodsFor: 'instance creation' stamp: 'nishis 5/15/98 01:14'!
new
	^super new initialize! !


!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/16/98 00:19'!
example1
	"Voronoi2dDiagram example1."

	| diagram gc random view |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	random := Random new.
	50 
		timesRepeat: 
			[diagram add: (random next @ random next scaleBy: diagram extent).
			diagram displayOn: gc.
			gc displayAt: view insetDisplayBox origin].
	Transcript cr; show: 'area: ' , diagram area rounded printString.
	gc displayAt: view insetDisplayBox origin.
	(Delay forMilliseconds: 1000) wait.
	view display.
	diagram show.
	^diagram! !

!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/16/98 00:21'!
example11
	"Voronoi2dDiagram example11."

	| diagram gc random view |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	diagram circumscriptionCircle: true.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	random := Random new.
	6 
		timesRepeat: 
			[diagram add: (random next @ random next scaleBy: diagram extent).
			diagram displayOn: gc.
			gc displayAt: view insetDisplayBox origin].
	Transcript cr; show: 'area: ' , diagram area rounded printString.
	gc displayAt: view insetDisplayBox origin.
	(Delay forMilliseconds: 1000) wait.
	view display.
	diagram show.
	^diagram! !

!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/16/98 00:32'!
example2
	"Voronoi2dDiagram example2."

	| diagram random scheduledView displayView displayView2 |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	random := Random new.
	50 timesRepeat: [diagram add: (random next @ random next scaleBy: diagram extent)].
	Transcript cr; show: 'area: ' , diagram area rounded printString.

	scheduledView := AlteredScheduledView
		model: diagram
		label: 'Voronoi & Delaunay Diagram'
		minimumSize: 400 @ 200 + 4
		maximumSize: 400 @ 200 + 4.
	displayView := DisplayView model: (DisplayModel visual: diagram diagram).
	displayView controller: NoController new.
	scheduledView addSubView: displayView.
	displayView2 := DisplayView model: (DisplayModel visual: diagram asDelaunayDiagram diagram).
	displayView2 controller: NoController new.
	scheduledView addSubView: displayView2 toRightOf: displayView.
	scheduledView controller openAt: 100 @ 100.
	^diagram! !

!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/24/98 08:53'!
example3
	"Voronoi2dDiagram example3."

	| string |
	string := 
'
	| diagram random window block process loop |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	random := Random new.
	window := ScheduledView new.
	window component: diagram.
	window label: diagram labelString.
	window minimumSize: diagram extent.
	window maximumSize: diagram extent.
	window openIn: (250 @ 50 extent: diagram extent).
	block := 
			[diagram add: (random next @ random next scaleBy: diagram extent).
			diagram displayOn: window graphicsContext.
			ScheduledControllers checkForEvents.
			Processor yield].
	process := nil.
	loop := true.
	diagram releaseBlock: [loop := false].
	process := [Object errorSignal
				handle: 
					[:exception | 
					process terminate.
					exception return]
				do: 
					[[loop]
						whileTrue: [block value]]] newProcess.
	process priority: Processor activeProcess priority - 1.
	process resume.
	^diagram'.

	StringHolderView open: (StringHolder new contents: string) label:  'Voronoi2dDiagram example3'.! !

!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/15/98 02:48'!
example4
	"This is a low precision example."
	"Voronoi2dDiagram example4."

	| diagram dots |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	dots := OrderedCollection new.
	dots add: 0.1 @ 0.1.
	dots add: 0.1 @ 0.9.
	dots add: 0.9 @ 0.1.
	dots add: 0.9 @ 0.9.
	dots add: 0.5 @ 0.5.
	dots add: 0.1 @ 0.5.
	dots add: 0.9 @ 0.5.
	dots add: 0.5 @ 0.1.
	dots add: 0.5 @ 0.9.
	dots do: [:dot | diagram add: (dot scaleBy: diagram extent)].
	Transcript cr; show: 'area: ' , diagram area rounded printString.
	diagram show.
	^diagram! !

!Voronoi2dDiagram class methodsFor: 'examples'!
example5
	"This is a low precision example."
	"Voronoi2dDiagram example5."

	| diagram count |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	count := 0.
	(0 to: 10 by: 1)
		do: [:y | (0 to: 10 by: 1)
				do: 
					[:x | 
					count even ifTrue: [diagram add: (x @ y / (10 @ 10) scaleBy: diagram extent)].
					count := count + 1]].
	Transcript cr; show: 'area: ' , diagram area rounded printString.
	diagram show.
	^diagram! !

!Voronoi2dDiagram class methodsFor: 'examples' stamp: 'nishis 5/15/98 02:52'!
example6
	"This is a low precision example."
	"Voronoi2dDiagram example6."

	| diagram dots step x y |
	diagram := Voronoi2dDiagram extent: 200 @ 200.
	dots := OrderedCollection new: 100.
	dots add: 0.0 @ 0.0.
	step := 10.
	(0 to: 360 by: step)
		do: 
			[:degrees | 
			x := degrees asFloat degreesToRadians cos.
			y := degrees asFloat degreesToRadians sin.
			dots add: x @ y.
			x := (degrees + (step / 2)) asFloat degreesToRadians cos.
			y := (degrees + (step / 2)) asFloat degreesToRadians sin.
			dots add: x @ y * (0.5 @ 0.5)].
	dots do: [:dot | diagram add: (dot + (1 @ 1) scaleBy: diagram extent / (2 @ 2))].
	Transcript cr; show: 'area: ' , diagram area rounded printString.
	diagram show.
	^diagram! !
