'Voronoi (11/18/2001 8:35:28 am)'.!


Voronoi2dObject subclass: #Voronoi2dProcessor
	instanceVariableNames: 'originalDots normalizedDots translationDelta scalingFactor voronoiDots voronoiPoints voronoiSides processingPrecision traceBoolean '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Voronoi-2d-Diagram'!

!Voronoi2dProcessor methodsFor: 'initialize-release'!
finalizeVoronoiDiagram
	| obstacles point side |
	obstacles := OrderedCollection new.
	1 to: voronoiPoints size
		do: 
			[:index | 
			point := voronoiPoints at: index.
			(point isInvalid and: [self startIndex <= index])
				ifTrue: [obstacles add: point]].
	obstacles do: [:theSide | voronoiPoints remove: theSide ifAbsent: [nil]].
	obstacles := OrderedCollection new.
	1 to: voronoiSides size
		do: 
			[:index | 
			side := voronoiSides at: index.
			(side isInvalid and: [self startIndex <= index])
				ifTrue: [obstacles add: side]].
	obstacles do: [:theSide | voronoiSides remove: theSide ifAbsent: [nil]]! !

!Voronoi2dProcessor methodsFor: 'initialize-release'!
initialize
	super initialize.
	normalizedDots := nil.
	translationDelta := nil.
	scalingFactor := nil.
	voronoiDots := nil.
	voronoiPoints := nil.
	voronoiSides := nil.
	processingPrecision := self accuracy.
	traceBoolean := false! !

!Voronoi2dProcessor methodsFor: 'initialize-release'!
initializeVoronoiDiagram
	| dot1 dot2 dot3 point1 point2 point3 line1 line2 point4 side1 side2 side3 |
	dot1 := Voronoi2dDot x: self bigSize negated y: self bigSize negated.
	dot2 := Voronoi2dDot x: 10.0 y: self bigSize.
	dot3 := Voronoi2dDot x: self bigSize y: self bigSize negated.
	point1 := Voronoi2dPoint x: dot1 x + dot2 x * 0.5 y: dot1 y + dot2 y * 0.5.
	point2 := Voronoi2dPoint x: dot2 x + dot3 x * 0.5 y: dot2 y + dot3 y * 0.5.
	point3 := Voronoi2dPoint x: dot3 x + dot1 x * 0.5 y: dot3 y + dot1 y * 0.5.
	line1 := dot1 bisector: dot2.
	line2 := dot2 bisector: dot3.
	point4 := line1 intersectingPointWithLine: line2.
	point1 := (Voronoi2dLine from: point1 to: point4)
				pointAtX: self bigNumber negated.
	point2 := (Voronoi2dLine from: point2 to: point4)
				pointAtX: self bigNumber.
	point3 := (Voronoi2dLine from: point3 to: point4)
				pointAtY: self bigNumber negated.
	side1 := Voronoi2dSide from: point1 to: point4.
	side1 dot1: dot1; dot2: dot2.
	side2 := Voronoi2dSide from: point2 to: point4.
	side2 dot1: dot2; dot2: dot3.
	side3 := Voronoi2dSide from: point3 to: point4.
	side3 dot1: dot3; dot2: dot1.
	point1 side1: side1.
	point2 side1: side2.
	point3 side1: side3.
	point4 side1: side1; side2: side2; side3: side3.
	voronoiDots := OrderedCollection new.
	voronoiPoints := OrderedCollection new.
	voronoiSides := OrderedCollection new.
	voronoiDots add: dot1; add: dot2; add: dot3.
	voronoiPoints add: point1; add: point2; add: point3; add: point4.
	voronoiSides add: side1; add: side2; add: side3.
	^self! !


!Voronoi2dProcessor methodsFor: 'accessing' stamp: 'nishis 6/28/1998 04:51'!
area
	| area |
	[area := 0.0.
	self triangles do: [:triangle | area := area + (Voronoi2dTriangle
						dot: (triangle at: 1)
						dot: (triangle at: 2)
						dot: (triangle at: 3)) area]]
		ifError: [:err :rcvr |  ^nil].
	^area! !

!Voronoi2dProcessor methodsFor: 'accessing'!
dots
	| stream |
	normalizedDots isNil ifTrue: [^Array new].
	stream := WriteStream on: (Array new: normalizedDots size).
	normalizedDots do: [:dot | stream nextPut: (self restoreDot: dot)].
	^stream contents! !

!Voronoi2dProcessor methodsFor: 'accessing' stamp: 'nishis 5/12/98 00:30'!
dots: dotCollection 
	| dots it |
	dots := OrderedCollection new: dotCollection size.
	dotCollection
		do: 
			[:dot | 
			it := dots detect: [:each | (each x - dot x) abs < self accuracy and: [(each y - dot y) abs < self accuracy]]
						ifNone: [nil].
			it isNil ifTrue: [dots add: (Voronoi2dDot x: dot x y: dot y)]].
	dots isEmpty ifTrue: [^nil].
	originalDots := dots asArray.
	normalizedDots := self normalizedDots: dots.
"self halt."! !

!Voronoi2dProcessor methodsFor: 'accessing'!
points
	| stream point |
	voronoiPoints isNil ifTrue: [^Array new].
	stream := WriteStream on: (Array new: voronoiPoints size).
	self startIndex to: voronoiPoints size
		do: 
			[:index | 
			point := voronoiPoints at: index.
			stream nextPut: (self restorePoint: point)].
	^stream contents! !

!Voronoi2dProcessor methodsFor: 'accessing'!
precision
	processingPrecision > (self accuracy * 10.0) ifTrue: [^#lowPrecision].
	^#highPrecision! !

!Voronoi2dProcessor methodsFor: 'accessing'!
sides
	| stream side |
	voronoiSides isNil ifTrue: [^Array new].
	stream := WriteStream on: (Array new: voronoiPoints size).
	self startIndex to: voronoiSides size
		do: 
			[:index | 
			side := voronoiSides at: index.
			stream nextPut: (self restoreSide: side)].
	^stream contents! !

!Voronoi2dProcessor methodsFor: 'accessing' stamp: 'nishis 5/12/98 03:23'!
triangles
	| stream point indexes p1 p2 p3 array |
	normalizedDots isNil ifTrue: [^Array new].
	stream := WriteStream on: (Array new: voronoiPoints size).
	self startIndex to: voronoiPoints size
		do: 
			[:index | 
			point := voronoiPoints at: index.
			indexes := self triangleIndexesAtPoint: point.
			indexes isNil
				ifFalse: 
					[p1 := self originalDots at: (indexes at: 1).
					p2 := self originalDots at: (indexes at: 2).
					p3 := self originalDots at: (indexes at: 3).
					array := Array
								with: (Point x: p1 x y: p1 y)
								with: (Point x: p2 x y: p2 y)
								with: (Point x: p3 x y: p3 y).
					stream nextPut: array]].
	^stream contents! !

!Voronoi2dProcessor methodsFor: 'accessing'!
trianglesUnderline
	| stream point triangle dots |
	normalizedDots isNil ifTrue: [^Array new].
	stream := WriteStream on: (Array new: voronoiPoints size).
	self startIndex to: voronoiPoints size
		do: 
			[:index | 
			point := voronoiPoints at: index.
			triangle := self triangleAtPoint: point.
			triangle isNil
				ifFalse: 
					[dots := Array
								with: triangle dot1
								with: triangle dot2
								with: triangle dot3.
					dots := dots collect: [:dot | self restoreDot: dot].
					triangle := Array
								with: (dots at: 1)
								with: (dots at: 2)
								with: (dots at: 3).
					stream nextPut: triangle]].
	^stream contents! !


!Voronoi2dProcessor methodsFor: 'basic accessing' stamp: 'nishis 5/12/98 03:38'!
dotsAtPoint: voronoi2dPoint 
	| side1 side2 side3 indexes index1 index2 index3 index4 index5 index6 dots |
	side1 := voronoi2dPoint side1.
	side2 := voronoi2dPoint side2.
	side3 := voronoi2dPoint side3.
	indexes := SortedCollection new: 6.
	indexes add: (voronoiDots indexOf: side1 dot1).
	indexes add: (voronoiDots indexOf: side1 dot2).
	indexes add: (voronoiDots indexOf: side2 dot1).
	indexes add: (voronoiDots indexOf: side2 dot2).
	indexes add: (voronoiDots indexOf: side3 dot1).
	indexes add: (voronoiDots indexOf: side3 dot2).
	index1 := indexes at: 1.
	index2 := indexes at: 2.
	index3 := indexes at: 3.
	index4 := indexes at: 4.
	index5 := indexes at: 5.
	index6 := indexes at: 6.
	index1 = index2 ifFalse: [^self error: 'unexpected error.'].
	index3 = index4 ifFalse: [^self error: 'unexpected error.'].
	index5 = index6 ifFalse: [^self error: 'unexpected error.'].
	dots := Array
				with: (voronoiDots at: index1)
				with: (voronoiDots at: index3)
				with: (voronoiDots at: index5).
	^dots! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
originalDots
	originalDots isNil ifTrue: [^Array new].
	^originalDots! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
processingPrecision
	^processingPrecision! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
restoreDot: voronoi2dDot 
	| x y dot |
	x := voronoi2dDot x * scalingFactor x + translationDelta x.
	y := voronoi2dDot y * scalingFactor y + translationDelta y.
	dot := Point x: x y: y.
	^dot! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
restorePoint: voronoi2dPoint 
	| x y point |
	x := voronoi2dPoint x * scalingFactor x + translationDelta x.
	y := voronoi2dPoint y * scalingFactor y + translationDelta y.
	point := Point x: x y: y.
	^point! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
restoreSide: voronoi2dSide 
	| from to side |
	from := self restorePoint: voronoi2dSide from.
	to := self restorePoint: voronoi2dSide to.
	side := Array with: from with: to.
	^side! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
scalingFactor
	^scalingFactor! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
startIndex
	^4! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
translationDelta
	^translationDelta! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
triangleAtPoint: voronoi2dPoint 
	| array index1 index3 index5 triangle |
	array := self dotsAtPoint: voronoi2dPoint.
	index1 := voronoiDots indexOf: (array at: 1).
	index3 := voronoiDots indexOf: (array at: 2).
	index5 := voronoiDots indexOf: (array at: 3).
	index1 < self startIndex ifTrue: [^nil].
	index3 < self startIndex ifTrue: [^nil].
	index5 < self startIndex ifTrue: [^nil].
	triangle := Voronoi2dTriangle
				dot: (voronoiDots at: index1)
				dot: (voronoiDots at: index3)
				dot: (voronoiDots at: index5).
	^triangle! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
triangleIndexesAtPoint: voronoi2dPoint 
	| array index1 index3 index5 indexes |
	array := self dotsAtPoint: voronoi2dPoint.
	index1 := voronoiDots indexOf: (array at: 1).
	index3 := voronoiDots indexOf: (array at: 2).
	index5 := voronoiDots indexOf: (array at: 3).
	index1 < self startIndex ifTrue: [^nil].
	index3 < self startIndex ifTrue: [^nil].
	index5 < self startIndex ifTrue: [^nil].
	indexes := Array
				with: index1 - self startIndex + 1
				with: index3 - self startIndex + 1
				with: index5 - self startIndex + 1.
	^indexes! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
voronoiDots
	^voronoiDots! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
voronoiPoints
	^voronoiPoints! !

!Voronoi2dProcessor methodsFor: 'basic accessing'!
voronoiSides
	^voronoiSides! !


!Voronoi2dProcessor methodsFor: 'computing' stamp: 'nishis 5/14/98 03:06'!
compute
	| condition |
	Cursor execute showWhile: 
		[condition := self computeFirst.
		condition ifFalse: [condition := self computeSecond]].
	^condition! !

!Voronoi2dProcessor methodsFor: 'computing' stamp: 'nishis 6/11/1998 22:45'!
computeFirst
	| gc index condition dot view |
	normalizedDots isNil ifTrue: [^true].
	self initializeVoronoiDiagram.
	self debugInitialize.
	self
		debug: 
			[:stream | 
			stream nextPutAll: Date today printString.
			stream space.
			stream nextPutAll: Time now printString].
	self trace
		ifTrue: 
			[view := ScheduledControllers activeController view.
			gc := Form extent: (view insetDisplayBox extent) depth: Display depth.
			self traceDisplayOn: gc].
	index := 1.
	condition := true.
	[condition and: [index <= normalizedDots size]]
		whileTrue: 
			[dot := normalizedDots at: index.
			self
				debug: 
					[:stream | 
					stream cr.
					stream nextPutAll: 'dots[' , (voronoiDots size + 1) printString.
					stream nextPutAll: ']: ' , dot printString].
			condition := self insertDot: dot.
			self trace ifTrue: [self traceDisplayOn: gc].
			index := index + 1]. 
	self trace
		ifTrue: 
			[Cursor crossHair showWhile: [Sensor waitClickButton]].
	self finalizeVoronoiDiagram.
	view = nil ifFalse: [view display].
	^condition! !

!Voronoi2dProcessor methodsFor: 'computing'!
computeSecond
	| random dot trembler x y condition |
	processingPrecision := 10.0 raisedTo: (self accuracy log / 2.0) rounded.
	random := Random new.
	1 to: normalizedDots size
		do: 
			[:index | 
			dot := normalizedDots at: index.
			trembler := random next - 0.5.
			x := 0.0 max: (dot x + (trembler * processingPrecision) min: 1.0).
			y := 0.0 max: (dot y + (trembler * processingPrecision) min: 1.0).
			dot x: x.
			dot y: y].
	condition := self computeFirst.
	condition ifFalse: [self insertError].
	^condition! !


!Voronoi2dProcessor methodsFor: 'inserting'!
bisectorSideBetween: voronoi2dDot1 and: voronoi2dDot2 
	| line point1 point2 |
	line := voronoi2dDot1 bisector: voronoi2dDot2.
	line isVertical
		ifTrue: 
			[point1 := line pointAtY: self bigNumber negated.
			point2 := line pointAtY: self bigNumber]
		ifFalse: 
			[point1 := line pointAtX: self bigNumber negated.
			point2 := line pointAtX: self bigNumber].
	^Voronoi2dSide from: point1 to: point2! !

!Voronoi2dProcessor methodsFor: 'inserting' stamp: 'nishis 5/14/98 02:04'!
insertDot: voronoi2dDot 
	| newPoints newSides obsoletePoints examinedSides rememberdEvaluations nearestDot bisectorSide1 anArray removedPoints removedSides |
	newPoints := OrderedCollection new.
	newSides := OrderedCollection new.
	obsoletePoints := IdentitySet new.
	examinedSides := IdentitySet new.
	rememberdEvaluations := OrderedCollection new.
	nearestDot := self nearestDot: voronoi2dDot.
	self debug: [:stream | stream cr; tab; nextPutAll: 'nearest point: ' , nearestDot printString].
	bisectorSide1 := self bisectorSideBetween: nearestDot and: voronoi2dDot.
	(self		
				searchSidesOfnearestDot: voronoi2dDot 
				bisectorSide1: bisectorSide1 
				nearestDot: nearestDot 
				examinedSides: examinedSides 
				obsoletePoints: obsoletePoints 
				rememberdEvaluations: rememberdEvaluations 
				newPoints: newPoints 
				newSides: newSides)
		ifFalse: [^false].
	rememberdEvaluations do: [:array | (array at: 1)
			perform: (array at: 2)
			with: (array at: 3)].
	(newPoints size > 0 and: [newSides size > 0])
		ifTrue: [(1 to: newPoints size)
				with: ((2 to: newSides size) asArray copyWith: 1)
				do: [:pointIndex :sideIndex | (newPoints at: pointIndex)
						side3: (newSides at: sideIndex)]].
	anArray := self
				removePoints: obsoletePoints
				andSides: examinedSides
				inside: newSides.
	removedPoints := anArray at: 1.
	removedSides := anArray at: 2.
	voronoiDots add: voronoi2dDot.
	newPoints do: [:point | (removedPoints includes: point)
			ifFalse: [voronoiPoints add: point]].
	newSides do: [:side | (removedSides includes: side)
			ifFalse: [voronoiSides add: side]].
	^true! !

!Voronoi2dProcessor methodsFor: 'inserting'!
insertError
	| errorBlock |
	errorBlock := 
			[:stream | 
			stream cr; nextPutAll: self class printString.
			stream nextPutAll: ' Error: infinite loop, '.
			stream nextPutAll: 'because all points on a lattice, '.
			stream nextPutAll: 'or no intersecting point, '.
			stream nextPutAll: 'or too many intersecting points.'.
			stream flush].
	self debug: [:stream | errorBlock value: stream].
	[errorBlock value: Transcript]
		forkAt: Processor activeProcess priority + 1.
	^false! !

!Voronoi2dProcessor methodsFor: 'inserting'!
nearerDot: voronoi2dDot except: nearestDot in: voronoi2dSide 
	| dotsAtPoint1 nearerDot |
	dotsAtPoint1 := self dotsAtPoint: voronoi2dDot.
	nearerDot := (dotsAtPoint1 select: [:dot | (dot = nearestDot) not and: [voronoi2dSide dot1 = dot or: [voronoi2dSide dot2 = dot]]]) first.
	^nearerDot! !

!Voronoi2dProcessor methodsFor: 'inserting'!
nearestDot: voronoi2dDot 
	| distance near d |
	distance := self bigNumber.
	near := nil.
	voronoiDots
		do: 
			[:dot | 
			d := voronoi2dDot distance: dot.
			distance > d
				ifTrue: 
					[distance := d.
					near := dot]].
	^near! !

!Voronoi2dProcessor methodsFor: 'inserting'!
removePoints: obsoletePoints andSides: examinedSides inside: newSides 
	| removedPoints removedSides point targetSide side |
	removedPoints := IdentitySet new.
	removedSides := IdentitySet new.
	obsoletePoints
		do: 
			[:obsoletePoint | 
			obsoletePoint side1: nil.
			obsoletePoint side2: nil.
			obsoletePoint side3: nil.
			point := voronoiPoints remove: obsoletePoint ifAbsent: [nil].
			point isNil ifFalse: [removedPoints add: obsoletePoint]].
	examinedSides do: [:examinedSide | ((obsoletePoints includes: examinedSide from)
			or: [obsoletePoints includes: examinedSide to])
			ifTrue: 
				[targetSide := newSides detect: [:newSide | (examinedSide intersectingPointWithSide: newSide) notNil]
							ifNone: [nil].
				targetSide isNil
					ifTrue: 
						[point := voronoiPoints remove: examinedSide from ifAbsent: [nil].
						point isNil ifFalse: [removedPoints add: point].
						point := voronoiPoints remove: examinedSide to ifAbsent: [nil].
						point isNil ifFalse: [removedPoints add: point].
						examinedSide from: nil.
						examinedSide to: nil.
						examinedSide dot1: nil.
						examinedSide dot2: nil.
						side := voronoiSides remove: examinedSide ifAbsent: [nil].
						side isNil ifFalse: [removedSides add: examinedSide]]
					ifFalse: [(examinedSide isInvalid and: [self startIndex <= (voronoiSides indexOf: examinedSide)])
							ifTrue: 
								[side := voronoiSides remove: examinedSide ifAbsent: [nil].
								side isNil ifFalse: [removedSides add: examinedSide]]]]
			ifFalse: [(examinedSide isInvalid and: [self startIndex <= (voronoiSides indexOf: examinedSide)])
					ifTrue: 
						[side := voronoiSides remove: examinedSide ifAbsent: [nil].
						side isNil ifFalse: [removedSides add: examinedSide]]]].
	^Array with: removedPoints with: removedSides! !

!Voronoi2dProcessor methodsFor: 'inserting'!
removingPoint: intersectingPoint with: voronoiDot1 with: voronoiDot2 with: voronoiSide 
	| triangle1 triangle2 point |
	triangle1 := Voronoi2dTriangle
				dot: intersectingPoint
				dot: voronoiDot1
				dot: voronoiDot2.
	triangle2 := Voronoi2dTriangle
				dot: intersectingPoint
				dot: voronoiDot1
				dot: voronoiSide from.
	triangle1 areaWithSign * triangle2 areaWithSign > 0
		ifTrue: [point := voronoiSide from]
		ifFalse: [point := voronoiSide to].
	^point! !

!Voronoi2dProcessor methodsFor: 'inserting'!
searchSides: voronoi2dDot 
	| sides obstacles side |
	sides := OrderedCollection new.
	obstacles := OrderedCollection new.
	1 to: voronoiSides size
		do: 
			[:index | 
			side := voronoiSides at: index.
			(side dot1 = voronoi2dDot or: [side dot2 = voronoi2dDot])
				ifTrue: [(side isInvalid and: [self startIndex <= index])
						ifTrue: [obstacles add: side]
						ifFalse: [sides add: side]]].
	obstacles do: [:theSide | voronoiSides remove: theSide ifAbsent: [nil]].
	^sides! !


!Voronoi2dProcessor methodsFor: 'private inserting' stamp: 'nishis 5/11/98 15:06'!
privateDebug: anObject with: aString
	self debug: [:stream | stream cr; tab; nextPutAll: aString , anObject printString].! !

!Voronoi2dProcessor methodsFor: 'private inserting' stamp: 'nishis 5/14/98 02:04'!
searchSidesOfnearestDot: voronoi2dDot bisectorSide1: bisectorSide1 nearestDot: nearestDot examinedSides: examinedSides obsoletePoints: obsoletePoints rememberdEvaluations: rememberdEvaluations newPoints: newPoints newSides: newSides
	| fromPoint newPoint1 toPoint oldPoint1 nearerDot newSide1 previousDot loopCounter bisectorSide2 newPoint2 oldPoint2 newSide2 |
	(self searchSides: nearestDot)
		do: [:side1 | (examinedSides includes: side1)
				ifFalse: 
					[fromPoint := nil.
					examinedSides add: side1.
					self privateDebug: side1 with: 'intersecting side: '.
					newPoint1 := side1 intersectingPointWithSide: bisectorSide1.
					self privateDebug: newPoint1 with: 'intersecting point: '.
					newPoint1 isNil
						ifFalse: 
							[toPoint := newPoint1.
							oldPoint1 := self
										removingPoint: newPoint1
										with: bisectorSide1 from
										with: voronoi2dDot
										with: side1.
							self privateDebug: oldPoint1 with: 'old point: '.
							obsoletePoints add: oldPoint1.
							nearerDot := self
										nearerDot: oldPoint1
										except: nearestDot
										in: side1.
							self privateDebug: nearerDot with: 'nearer point: '.
							newSide1 := Voronoi2dSide from: fromPoint to: toPoint.
							newSide1 dot1: voronoi2dDot.
							newSide1 dot2: nearestDot.
							previousDot := nearerDot.
							loopCounter := 1.
							[nearestDot = nearerDot]
								whileFalse: 
									[bisectorSide2 := self bisectorSideBetween: nearerDot and: voronoi2dDot.
									(self searchSides: nearerDot)
										do: [:side2 | (examinedSides includes: side2)
												ifFalse: 
													[fromPoint := toPoint.
													examinedSides add: side2.
													self privateDebug: side2 with: 'intersecting side: '.
													newPoint2 := side2 intersectingPointWithSide: bisectorSide2.
													self privateDebug: side2 with: 'intersecting point: '.
													newPoint2 isNil
														ifFalse: 
															[toPoint := newPoint2.
															oldPoint2 := self
																		removingPoint: newPoint2
																		with: bisectorSide2 from
																		with: voronoi2dDot
																		with: side2.
															self privateDebug: oldPoint2 with: 'old point: '.
															obsoletePoints add: oldPoint2.
															nearerDot := self
																		nearerDot: oldPoint2
																		except: nearerDot
																		in: side2.
															self privateDebug: nearerDot with: 'nearer point: '.
															newSide2 := Voronoi2dSide from: fromPoint to: toPoint.
															newSide2 dot1: voronoi2dDot.
															newSide2 dot2: previousDot.
															side2 from = oldPoint2
																ifTrue: [rememberdEvaluations add: (Array
																			with: side2
																			with: #from:
																			with: newPoint2)]
																ifFalse: [rememberdEvaluations add: (Array
																			with: side2
																			with: #to:
																			with: newPoint2)].
															newPoint2 side1: side2.
															newPoint2 side2: newSide2.
															previousDot := nearerDot.
															newPoints add: newPoint2.
															newSides add: newSide2]]].
									loopCounter := loopCounter + 1.
									loopCounter > self loopMax ifTrue: [^false].
									Processor yield].
							side1 from = oldPoint1
								ifTrue: [rememberdEvaluations add: (Array
											with: side1
											with: #from:
											with: newPoint1)]
								ifFalse: [rememberdEvaluations add: (Array
											with: side1
											with: #to:
											with: newPoint1)].
							newSide1 from: toPoint.
							newPoint1 side1: side1.
							newPoint1 side2: newSide1.
							newPoints add: newPoint1.
							newSides add: newSide1]]].
	^true! !


!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/12/98 00:58'!
displayBorderOn: graphicsContext box: aRectangle lineWidth: lineWidth color: aColor 
	| quadrangle |
	quadrangle := Quadrangle region: aRectangle 
			borderWidth: lineWidth 
			borderColor: aColor 
			insideColor: nil.
	quadrangle displayOn: graphicsContext! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/16/98 00:38'!
displayBoxOn: graphicsContext box: aRectangle clip: clippingBox lineWidth: lineWidth color: aColor 
	| quadrangle |
	(aRectangle intersects: clippingBox)
		ifTrue: 
			[quadrangle := Quadrangle region: aRectangle 
					borderWidth: lineWidth 
					borderColor: aColor 
					insideColor: aColor.
			quadrangle displayOn: graphicsContext]! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/16/98 00:38'!
displayCircleOn: graphicsContext box: box lineWidth: lineWidth color: aColor 
	| aCircle aForm |
	aCircle := Circle new.
	aForm := Form extent: lineWidth @ lineWidth depth: Display depth.
	aForm fillColor: aColor.
	aCircle form: aForm.
	aCircle radius: box height / 2.
	aCircle center: box center.
	aCircle displayOn: graphicsContext! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/16/98 00:38'!
displayCircleOn: graphicsContext radius: raduis center: center lineWidth: lineWidth color: aColor 
	| aCircle aForm |
	aCircle := Circle new.
	aForm := Form extent: lineWidth @ lineWidth depth: Display depth.
	aForm fillColor: aColor.
	aCircle form: aForm.
	aCircle radius: raduis.
	aCircle center: center.
	aCircle displayOn: graphicsContext ! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/20/98 22:21'!
displayLineOn: graphicsContext from: fromPoint to: toPoint clip: clippingBox lineWidth: lineWidth color: aColor 
	| slope bounds boolean1 boolean2 boolean3 side line point1 point2 point aForm aLine |
	aForm := Form extent: lineWidth @ lineWidth depth: Display depth.
	aForm fillColor: aColor.
	aLine := Line new.
	aLine form: aForm.

	slope := ((toPoint y - fromPoint y) abs / ((toPoint x - fromPoint x) abs max: self accuracy)) asFloat.
	bounds := clippingBox.
	bounds := bounds expandBy: (0 @ 0 corner: 1 @ 1).
	boolean1 := bounds containsPoint: fromPoint.
	boolean2 := bounds containsPoint: toPoint.
	bounds := bounds insetBy: (0 @ 0 corner: 1 @ 1).
	(boolean1 and: [boolean2])
		ifTrue: [aLine beginPoint: fromPoint; endPoint: toPoint; displayOn: graphicsContext.
				"graphicsContext displayLineFrom: fromPoint to: toPoint"]
		ifFalse: [(boolean1 not and: [boolean2 not])
				ifTrue: 
					[boolean3 := false.
					side := Voronoi2dSide from: fromPoint to: toPoint.
					((Voronoi2dSide from: bounds topLeft to: bounds bottomLeft)
						intersectingPointWithSide: side) isNil ifFalse: [boolean3 := true].
					((Voronoi2dSide from: bounds topRight to: bounds bottomRight)
						intersectingPointWithSide: side) isNil ifFalse: [boolean3 := true].
					((Voronoi2dSide from: bounds topLeft to: bounds topRight)
						intersectingPointWithSide: side) isNil ifFalse: [boolean3 := true].
					((Voronoi2dSide from: bounds bottomLeft to: bounds bottomRight)
						intersectingPointWithSide: side) isNil ifFalse: [boolean3 := true].
					boolean3
						ifTrue: 
							[line := Voronoi2dLine from: fromPoint to: toPoint.
							slope > 1.0
								ifTrue: 
									[point1 := line pointAtY: bounds top.
									point2 := line pointAtY: bounds bottom]
								ifFalse: 
									[point1 := line pointAtX: bounds left.
									point2 := line pointAtX: bounds right].
							point1 := point1 x rounded @ point1 y rounded.
							point2 := point2 x rounded @ point2 y rounded.
							aLine beginPoint: point1; endPoint: point2; displayOn: graphicsContext]]
				ifFalse: 
					[boolean1
						ifTrue: 
							[point1 := fromPoint.
							point2 := toPoint.
							side := Voronoi2dSide from: fromPoint to: toPoint.
							(point := (Voronoi2dSide from: bounds topLeft to: bounds bottomLeft)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds topRight to: bounds bottomRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds topLeft to: bounds topRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds bottomLeft to: bounds bottomRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point]]
						ifFalse: 
							[point1 := toPoint.
							point2 := fromPoint.
							side := Voronoi2dSide from: fromPoint to: toPoint.
							(point := (Voronoi2dSide from: bounds topLeft to: bounds bottomLeft)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds topRight to: bounds bottomRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds topLeft to: bounds topRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point].
							(point := (Voronoi2dSide from: bounds bottomLeft to: bounds bottomRight)
										intersectingPointWithSide: side) isNil ifFalse: [point2 := point]].
					point1 := point1 x rounded @ point1 y rounded.
					point2 := point2 x rounded @ point2 y rounded.
					aLine beginPoint: point1; endPoint: point2; displayOn: graphicsContext]]! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/12/98 01:12'!
displayLoopOn: graphicsContext points: pointCollection clip: clippingBox lineWidth: lineWidth color: aColor 
	| fromPoint toPoint |
	pointCollection size < 3 ifTrue: [^self].
	(1 to: pointCollection size - 1)
		do: 
			[:index | 
			fromPoint := pointCollection at: index.
			toPoint := pointCollection at: index + 1.
			self
				displayLineOn: graphicsContext
				from: fromPoint
				to: toPoint
				clip: clippingBox
				lineWidth: lineWidth 
				color: aColor].
	pointCollection first = pointCollection last ifFalse: [self
			displayLineOn: graphicsContext
			from: pointCollection first
			to: pointCollection last
			clip: clippingBox
			lineWidth: lineWidth 
			color: aColor]! !

!Voronoi2dProcessor methodsFor: 'displaying' stamp: 'nishis 5/12/98 01:48'!
displayPolylineOn: graphicsContext points: points lineWidth: lineWidth color: aColor 
	| aLinearFit aForm |
	aLinearFit := LinearFit new.
	aForm := Form extent: lineWidth @ lineWidth.
	aForm fillColor: aColor.
	aLinearFit form: aForm.
	points do: [:point | aLinearFit add: point].
	aLinearFit displayOn: graphicsContext! !


!Voronoi2dProcessor methodsFor: 'printing'!
printOn: aStream 
	aStream nextPutAll: 'diagram ('.
	aStream cr.
	self dots
		do: 
			[:dot | 
			aStream tab.
			dot printOn: aStream.
			aStream cr].
	aStream nextPutAll: ')'.
	aStream cr! !


!Voronoi2dProcessor methodsFor: 'tracing'!
trace
	^traceBoolean = true! !

!Voronoi2dProcessor methodsFor: 'tracing'!
trace: aBoolean 
	traceBoolean := aBoolean = true! !

!Voronoi2dProcessor methodsFor: 'tracing' stamp: 'nishis 5/14/98 02:49'!
traceDisplayOn1: graphicsContext 
	| traceBlock bounds origin factor gc box sides point1 point2 |
	traceBlock := 
			[bounds := graphicsContext boundingBox.
			origin := bounds center.
			factor := 200 @ 200.
			origin := origin - (0.5 @ 0.5 * factor).
			"pixmap := pixmap extent: bounds extent."
			gc := "pixmap" graphicsContext.
			gc fillWithColor: Color white.
			gc borderWidth: 2 color: Color black.
			voronoiDots
				do: 
					[:dot | 
					box := (dot x @ dot y * factor + origin) rounded extent: 1 @ 1.
					box := box expandBy: 2.
					self
						displayBoxOn: gc
						box: box
						clip: bounds 
						lineWidth: 1 
						color: Color red].
			voronoiPoints
				do: 
					[:point | 
					box := (point x @ point y * factor + origin) rounded extent: 1 @ 1.
					box := box expandBy: 3.
					self 
						displayBoxOn: gc 
						box: box 
						clip: box 
						lineWidth: 1 
						color: Color green.
					sides := OrderedCollection new: 3.
					point side1 isNil ifFalse: [sides add: point side1].
					point side2 isNil ifFalse: [sides add: point side2].
					point side3 isNil ifFalse: [sides add: point side3].
					sides
						do: 
							[:side | 
							point1 := (side from x @ side from y * factor + origin) rounded.
							point2 := (side to x @ side to y * factor + origin) rounded.
							self
								displayLineOn: gc
								from: point1
								to: point2
								clip: bounds
								lineWidth: 3 
								color: Color blue]].
			voronoiSides
				do: 
					[:side | 
					point1 := (side from x @ side from y * factor + origin) rounded.
					point2 := (side to x @ side to y * factor + origin) rounded.
					self
						displayLineOn: gc
						from: point1
						to: point2
						clip: bounds 
						lineWidth: 1 
						color: Color cyan].
			self 
				displayBorderOn: gc 
				box: (((0 @ 0 extent: 1 @ 1)
							scaleBy: factor)
							translateBy: origin) 
				lineWidth: 1 
				color: Color black.
			gc displayAt: ScheduledControllers activeController view insetDisplayBox origin].

	[traceBlock value] 
		ifError:  
			[:msg :rec | 
			self debug: [Transcript cr; show: 'error in trace display'].
			^nil]
! !

!Voronoi2dProcessor methodsFor: 'tracing' stamp: 'nishis 5/14/98 02:58'!
traceDisplayOn: graphicsContext 
	| bounds origin factor "pixmap" gc box point1 point2 sides |
	[bounds := graphicsContext boundingBox.
	origin := bounds center.
	factor := 200 @ 200.
	origin := origin - (0.5 @ 0.5 * factor).
	"pixmap := pixmap extent: bounds extent."
	gc := "pixmap" graphicsContext.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	voronoiDots
		do: 
			[:dot | 
			box := (dot x @ dot y * factor + origin) rounded extent: 1 @ 1.
			box := box expandBy: 2.
			"Transcript cr; show: 'box: ', box asString; tab; show: 'dot: ', dot asString."
			self
				displayBoxOn: gc
				box: box
				clip: bounds 
				lineWidth: 1 
				color: Color red].
	voronoiPoints
		do: 
			[:point | 
			box := (point x @ point y * factor + origin) rounded extent: 1 @ 1.
			box := box expandBy: 3.
			self displayBoxOn: gc box: box clip: box lineWidth: 1 color: Color green.
			sides := OrderedCollection new: 3.
			point side1 isNil ifFalse: [sides add: point side1].
			point side2 isNil ifFalse: [sides add: point side2].
			point side3 isNil ifFalse: [sides add: point side3].
			sides
				do: 
					[:side | 
					point1 := (side from x @ side from y * factor + origin) rounded.
					point2 := (side to x @ side to y * factor + origin) rounded.
					self
						displayLineOn: gc
						from: point1
						to: point2
						clip: bounds
						lineWidth: 3 
						color: Color blue]].
	voronoiSides
		do: 
			[:side | 
			point1 := (side from x @ side from y * factor + origin) rounded.
			point2 := (side to x @ side to y * factor + origin) rounded.
			self 
				displayLineOn: gc 
				from: point1 
				to: point2 
				clip: bounds 
				lineWidth: 1 
				color: Color cyan].
	self 
		displayBorderOn: gc 
		box: (((0 @ 0 extent: 1 @ 1)
					scaleBy: factor)
					translateBy: origin) 
		lineWidth: 1 
		color: Color black.
	gc displayAt: ScheduledControllers activeController view insetDisplayBox origin]
		ifError:
			[:msg :rec | 
			self debug: [Transcript cr; show: 'error in trace display']]! !


!Voronoi2dProcessor methodsFor: 'private'!
bigFactor
	^100.0! !

!Voronoi2dProcessor methodsFor: 'private'!
bigNumber
	^self bigSize * self bigFactor! !

!Voronoi2dProcessor methodsFor: 'private'!
bigSize
	^100.0! !

!Voronoi2dProcessor methodsFor: 'private'!
loopMax
	^5000! !

!Voronoi2dProcessor methodsFor: 'private'!
normalizedDots: dotCollection 
	| dots minX minY maxX maxY delta width height factor |
	dots := dotCollection asArray.
	minX := nil.
	minY := nil.
	maxX := nil.
	maxY := nil.
	dots
		do: 
			[:dot | 
			minX isNil
				ifTrue: [minX := dot x]
				ifFalse: [minX := dot x min: minX].
			minY isNil
				ifTrue: [minY := dot y]
				ifFalse: [minY := dot y min: minY].
			maxX isNil
				ifTrue: [maxX := dot x]
				ifFalse: [maxX := dot x max: maxX].
			maxY isNil
				ifTrue: [maxY := dot y]
				ifFalse: [maxY := dot y max: maxY]].
	((0.0 <= minX and: [maxX <= 1.0])
		and: [0.0 <= minY and: [maxY <= 1.0]])
		ifTrue: 
			[translationDelta := Voronoi2dDot x: 0.0 y: 0.0.
			scalingFactor := Voronoi2dDot x: 1.0 y: 1.0.
			^dots].
	delta := Voronoi2dDot x: minX y: minY.
	dots := dots collect: [:dot | Voronoi2dDot x: dot x - delta x y: dot y - delta y].
	width := maxX - minX max: self accuracy.
	height := maxY - minY max: self accuracy.
	factor := width max: height.
	factor := Voronoi2dDot x: factor y: factor.
	dots := dots collect: [:dot | Voronoi2dDot x: dot x / factor x y: dot y / factor y].
	translationDelta := delta.
	scalingFactor := factor.
	^dots! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Voronoi2dProcessor class
	instanceVariableNames: ''!

!Voronoi2dProcessor class methodsFor: 'copyright'!
copyright
	^'Copyright 1991-1998 AOKI Atsushi'! !

!Voronoi2dProcessor class methodsFor: 'copyright'!
system
	^'Voronoi and Delaunay Diagram'! !

!Voronoi2dProcessor class methodsFor: 'copyright'!
version
	^'006'! !


!Voronoi2dProcessor class methodsFor: 'instance creation' stamp: 'nishis 5/11/98 04:11'!
dots: dotCollection 
	^(super new) dots: dotCollection; yourself! !


!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/11/98 04:16'!
example1
	"Voronoi2dProcessor example1."

	| dots voronoi |
	dots := OrderedCollection new.
	dots add: 0.0 @ 0.0.
	dots add: 0.1 @ 1.0.
	dots add: 1.0 @ 0.1.
	dots add: 1.0 @ 1.0.
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/12/98 02:07'!
example2
	"Voronoi2dProcessor example2."

	| dots voronoi triangles gc scale translation box points view |
	dots := OrderedCollection new.
	dots add: -1.0709950047146 @ -1.0.
	dots add: -1.3616787517522 @ -0.55623057513983.
	dots add: -1.4749357023933 @ 1.1021456892432e-16.
	dots add: -1.3616787517522 @ 0.55623057513983.
	dots add: -1.0329942792654 @ 1.0580134260987.
	dots add: -0.83909963117728 @ 1.2088366880644.
	dots add: -0.3639702342662 @ 1.4500662049207.
	dots add: -0.3290104887271 @ 1.456230551298.
	dots add: 0.0 @ 1.4857898683076.
	dots add: 0.3290104887271 @ 1.456230551298.
	dots add: 0.3639702342662 @ 1.4500662049207.
	dots add: 0.83909963117728 @ 1.2088366880644.
	dots add: 1.0 @ 1.0836784193003.
	dots add: 1.0 @ 3.0.
	dots add: -3.0 @ 3.0.
	dots add: -3.0 @ -1.0.
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 50 @ 50.
	translation := 200 @ 100.
	box := ((-3 @ -1 extent: 4 @ 4)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.

	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/12/98 02:07'!
example3
	"Voronoi2dProcessor example3."

	| random dots voronoi triangles gc scale translation box points view |
	random := Random new.
	dots := OrderedCollection new.
	50 timesRepeat: [dots add: random next @ random next].
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 200 @ 200.
	translation := 50 @ 50.
	box := ((0 @ 0 extent: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/12/98 03:32'!
example4
	"Voronoi2dProcessor example4."

	| dots voronoi x y triangles gc scale translation box points view |
	dots := OrderedCollection new.
	dots add: 0.0 @ 0.0.
	(0 to: 360 by: 10)
		do: 
			[:degrees | 
			x := degrees asFloat degreesToRadians cos.
			y := degrees asFloat degreesToRadians sin.
			dots add: x @ y].
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 100 @ 100.
	translation := 150 @ 150.
	box := ((-1 @ -1 corner: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/12/98 02:11'!
example5
	"Voronoi2dProcessor example5."

	| stream dots voronoi triangles gc scale translation box points strm x y view |
	stream := ReadStream on: '
0.20254893600941 0.30189919471741
0.90037482976913 0.97756767272949
0.37021240592003 0.7721865773201
0.73451060056686 0.66534775495529
0.83000415563583 0.35092878341675
0.20771345496178 0.23709288239479
0.13294461369514 0.44628903269768
0.585289478302 0.81525200605392
0.25862556695938 0.89500206708908
0.56111621856689 0.95084547996521
0.61521035432816 0.96237403154373
0.25764265656471 0.62346524000168
0.039433568716049 0.34228238463402
0.22588920593262 0.24127447605133
0.79660141468048 0.157176181674
0.445447742939 0.64100790023804
0.58059144020081 0.95807582139969
0.32448145747185 0.087971679866314
0.51385253667831 0.72457307577133
0.69972509145737 0.27604332566261
0.14740525186062 0.86483132839203
0.83648478984833 0.32767179608345
0.46084132790565 0.21558517217636
0.42863807082176 0.79387754201889
0.86232405900955 0.042140774428844
0.59863388538361 0.654252409935
0.18858808279037 0.038375675678253
0.89749270677567 0.31695958971977
0.70493960380554 0.32673886418343
0.30480632185936 0.61873388290405
0.10525614023209 0.45825073122978
0.22873802483082 0.82488131523132
0.51198667287827 0.41250312328339
0.50602251291275 0.62927949428558
0.47615161538124 0.59765928983688
0.40196585655212 0.15437734127045
0.97734278440475 0.85045397281647
0.61044561862946 0.95119535923004
0.42372345924377 0.43619325757027
0.25391086935997 0.99932527542114
0.65202832221985 0.048754684627056
0.88338190317154 0.70693045854568
0.92756348848343 0.81058722734451
0.47845065593719 0.91074550151825
0.94420659542084 0.27271136641502
0.44728028774261 0.87607663869858
0.82440650463104 0.41471886634827
0.62660557031631 0.29680132865906
0.11918367445469 0.64477300643921
0.28173261880875 0.29536858201027
0.808129966259 0.7996084690094
0.10653894394636 0.4227988421917
0.29940858483315 0.1445314437151
0.22347354888916 0.65868389606476
0.42975425720215 0.37341940402985
0.18355685472488 0.4111869931221
0.46447312831879 0.60872137546539
0.96638065576553 0.51704287528992
0.097442731261253 0.40145772695541
0.9801082611084 0.24155768752098
0.45111203193665 0.73122030496597
0.06147438660264 0.27860891819
0.076501458883286 0.0061724283732474
0.47578507661819 0.75064557790756
0.95320284366608 0.063048727810383
0.91691797971725 0.2086880505085
0.48937943577766 0.16715535521507
0.5073219537735 0.6323282122612
0.52176594734192 0.012369845993817
0.79801750183105 0.42972928285599
0.31566846370697 0.7211412191391
0.76859641075134 0.43762597441673
0.56496459245682 0.84448981285095
0.82722198963165 0.92132443189621
0.3921032845974 0.36200749874115
0.81062889099121 0.57470220327377
0.34810495376587 0.68185758590698
0.98412328958511 0.52020823955536
0.4125614464283 0.64077466726303
0.041582673788071 0.30886298418045
0.99363601207733 0.50406497716904
0.10545606166124 0.92025822401047
0.92806333303452 0.96560597419739
0.72676384449005 0.7625572681427
0.4811495244503 0.14784672856331
0.88509786128998 0.67249476909637
0.34677216410637 0.60180759429932
0.98862141370773 0.91537690162659
0.64738023281097 0.30708038806915
0.87406915426254 0.18508121371269
0.9339941740036 0.67182838916779
0.80674719810486 0.60405665636063
0.18620574474335 0.53278630971909
0.48053312301636 0.72332364320755
0.8121782541275 0.15526030957699
0.017875885590911 0.52247399091721
0.64864641427994 0.23312786221504
0.96979594230652 0.40967097878456
0.96091628074646 0.88883799314499
0.31588503718376 0.22164931893349
0.44286546111107 0.67341107130051
0.46430653333664 0.22371511161327
0.21694293618202 0.56643897294998
0.25179508328438 0.10974594205618
0.83416908979416 0.97608494758606
0.94365680217743 0.0021907538175583
0.27413576841354 0.73908370733261
0.23376926779747 0.45206996798515
0.94500625133514 0.12074135988951
0.24458143115044 0.43897542357445
0.68351519107819 0.81493544578552
0.52711367607117 0.37107038497925
0.75496876239777 0.94411492347717
0.060957934707403 0.085089549422264
0.85324448347092 0.059300292283297
0.25427737832069 0.84633904695511
0.1007913351059 0.14008329808712
0.94380676746368 0.34869638085365
0.048629738390446 0.5946272611618
0.39485213160515 0.71461057662964
0.67635154724121 0.25966680049896
0.30129113793373 0.49510204792023
0.39211994409561 0.40050813555717
0.78557270765305 0.66976261138916
0.032669719308615 0.71102875471115
0.39871719479561 0.64675551652908
0.8633069396019 0.31367763876915
0.12031653523445 0.26281550526619
0.57790923118591 0.75947523117065
0.3585339486599 0.78324866294861
0.29892545938492 0.02801332809031
0.95010411739349 0.90193253755569
0.5773760676384 0.52745521068573
0.16033320128918 0.74131613969803
0.39285299181938 0.094535611569881
0.68309873342514 0.85241985321045
0.15351936221123 0.99456059932709
0.64084964990616 0.21483547985554
0.69610995054245 0.92140775918961
0.58460640907288 0.23672635853291
0.2859308719635 0.99752604961395
0.49396085739136 0.75482714176178
0.61685961484909 0.77393585443497
0.77707624435425 0.034443981945515
0.81134527921677 0.2302290648222
0.27068722248077 0.76945441961288
0.4204081594944 0.77456891536713
0.24009995162487 0.082307368516922
0.4236401617527 0.24369013309479
0.37919199466705 0.52402329444885
0.22920449078083 0.90289878845215
0.81041234731674 0.074194088578224
0.67383587360382 0.44607245922089
0.084781341254711 0.14098292589188
0.022840483114123 0.99566012620926
0.18189087510109 0.56112450361252
0.97009581327438 0.10268221795559
0.5099042057991 0.59992504119873
0.63805079460144 0.74673050642014
0.90555602312088 0.9512619972229
0.57772594690323 0.33596834540367
0.63415241241455 0.73758435249329
0.76871305704117 0.70713037252426
0.38957101106644 0.50991255044937
0.61917531490326 0.12552270293236
0.29426071047783 0.2478384077549
0.96584755182266 0.28502291440964
0.89924198389053 0.35952520370483
0.074035815894604 0.30807995796204
0.18410661816597 0.68170762062073
0.63761764764786 0.74571430683136
0.55701792240143 0.47969180345535
0.77904206514359 0.57751768827438
0.85471051931381 0.44735527038574
0.049329444766045 0.21165347099304
0.34249064326286 0.70714700222015
0.42807164788246 0.48485630750656
0.71423572301865 0.81008744239807
0.32343190908432 0.66243231296539
0.092394836246967 0.73576843738556
0.57214492559433 0.43825906515121
0.027988338842988 0.89235317707062
0.43951687216759 0.93478548526764
0.500608086586 0.11657642573118
0.6194252371788 0.7030320763588
0.91841733455658 0.6737442612648
0.23431903123856 0.72259056568146
0.11815077066422 0.25773426890373
0.83521866798401 0.40162432193756
0.36511453986168 0.99099540710449
0.4017159640789 0.57686799764633
0.353186160326 0.42454811930656
0.34197416901588 0.51362764835358
0.20481465756893 0.53798419237137
0.49272802472115 0.90578091144562
0.47102040052414 0.73946690559387
0.11928363144398 0.87577676773071
0.13139525055885 0.86573094129562
0.91551852226257 0.97463554143906
0.59410244226456 0.18208245933056
0.0038817159365863 0.18194918334484
0.69587671756744 0.38239899277687
0.93539357185364 0.90588086843491
0.70202416181564 0.58912950754166
0.68964600563049 0.98316532373428
0.30642232298851 0.35329446196556
0.67480218410492 0.67910867929459
0.63152021169662 0.65448564291
0.72759681940079 0.68758851289749
0.22833819687366 0.90086632966995
0.11333610862494 0.13105373084545
0.076468139886856 0.92917120456696
0.52589756250381 0.56052476167679
0.58407330513 0.0047063725069165
0.08773010969162 0.95559352636337
0.5878883600235 0.82134944200516
0.34985423088074 0.72442317008972
0.35321947932243 0.50154936313629
0.29186171293259 0.70374846458435
0.57394421100616 0.59632652997971
0.32191586494446 0.15887546539307
0.37251144647598 0.085272803902626
0.27675133943558 0.78368180990219
0.29994168877602 0.37655144929886
0.42167431116104 0.70061641931534
0.33582672476768 0.30689713358879
0.45056226849556 0.46069970726967
0.88832986354828 0.14161598682404
0.48586422204971 0.043523531407118
0.79418575763702 0.57458561658859
0.078600585460663 0.85725116729736
0.31870055198669 0.72825491428375
0.20836317539215 0.7386172413826
0.15575176477432 0.15364430844784
0.28331527113914 0.95292794704437
0.42778840661049 0.8303456902504
0.14019158482552 0.19406080245972
0.68583089113235 0.16652227938175
0.044298209249973 0.58446478843689
0.90947103500366 0.9989088177681
0.68951272964478 0.67516034841537
0.50687211751938 0.59281134605408
0.19828404486179 0.44573926925659
0.31476885080338 0.64210742712021
0.12163265049458 0.30436486005783
0.59846729040146 0.26924616098404
0.43915036320686 0.087771758437157
0.051845066249371 0.025247814133763
0.5590004324913 0.061266139149666
0.79735112190247 0.88970428705215
0.31791752576828 0.91872555017471
0.38600581884384 0.27077883481979
0.98120784759521 0.78259891271591
0.79740107059479 0.0052061639726162
0.24274885654449 0.20390670001507
0.43970012664795 0.35829237103462
0.22498959302902 0.16224072873592
0.14963765442371 0.023915035650134
0.47895044088364 0.065764263272285
0.19251978397369 0.12452311813831
0.9842232465744 0.75121200084686
0.26222407817841 0.21113702654839
0.14897125959396 0.48389005661011
0.48119950294495 0.26334860920906
0.80992919206619 0.95767599344254
0.40046647191048 0.68932110071182
0.23240315914154 0.29501873254776
0.99961680173874 0.32578924298286
0.11027072370052 0.04693877696991
0.68681383132935 0.43805915117264
0.56598085165024 0.19302789866924
0.2987921833992 0.72000831365585
0.15055394172668 0.14144939184189
0.10085797309875 0.29408580064774
0.84358185529709 0.72893792390823
0.78688877820969 0.71131193637848
0.053227823227644 0.22079966962337
0.47933360934258 0.95127862691879
0.61622655391693 0.31091213226318
0.72921282052994 0.422149091959
0.79788422584534 0.12172428518534
0.51611828804016 0.96065807342529
0.29207828640938 0.20425656437874
0.24821324646473 0.83211159706116
0.22125780582428 0.53810077905655
0.76223242282867 0.73038733005524
0.13644315302372 0.53142023086548
0.32348188757896 0.77793419361114
0.017226155847311 0.02094960398972
0.62583923339844 0.52577257156372
0.27177008986473 0.27199500799179
0.79175341129303 0.95349436998367
0.73680967092514 0.97843396663666
0.37224489450455 0.46926280856133
0.67765098810196 0.2627155482769
0.34690546989441 0.90981256961823
0.78817158937454 0.67586004734039
0.1238983720541 0.54044979810715
0.19082048535347 0.197459384799
0.53995835781097 0.055052060633898
0.4366180896759 0.23567679524422
0.860391497612 0.57606828212738
0.50515615940094 0.62724697589874
0.77907538414001 0.65451896190643
0.80459809303284 0.63747602701187
0.41845896840096 0.2699958384037
0.17167846858501 0.9602415561676
0.3295626938343 0.8306622505188
0.87170344591141 0.71799248456955
0.49197834730148 0.17325280606747
0.59855061769485 0.46174928545952
0.31386920809746 0.56307369470596
0.47466889023781 0.17110370099545
0.6319699883461 0.69400250911713
0.051078718155622 0.2542190849781
0.71158683300018 0.68848812580109
0.30737191438675 0.54783004522324
0.24659724533558 0.097551017999649
0.65171176195145 0.31724280118942
0.35945022106171 0.90078300237656
0.92083299160004 0.2563348710537
0.60116618871689 0.50634735822678
0.38004165887833 0.48755517601967
0.95133692026138 0.75097876787186
0.72321534156799 0.56192421913147
0.81812578439713 0.89998334646225
0.072802998125553 0.45903372764587
0.038267388939857 0.64723867177963
0.97982507944107 0.58704704046249
0.87706786394119 0.1151936724782
0.42387339472771 0.7826988697052
0.028404831886292 0.85486882925034
0.81311118602753 0.31129530072212
0.61472719907761 0.84585589170456
0.98427319526672 0.86671388149261
0.18705539405346 0.4963181912899
0.20266555249691 0.57140356302261
0.72498124837875 0.64299041032791
0.16216576099396 0.97638481855392
0.6366680264473 0.55117869377136
0.98522281646729 0.061249479651451
0.75885045528412 0.91476052999496
0.22285714745522 0.23416076600552
0.35683465003967 0.85618489980698
0.85466057062149 0.33185338973999
0.12449812889099 0.92647230625153
0.28879633545876 0.61963349580765
0.18428987264633 0.10521449148655
0.36199915409088 0.79137861728668
0.087230317294598 0.8005747795105
0.33957517147064 0.96953767538071
0.81291127204895 0.84928780794144
0.91540193557739 0.70513117313385
0.76949602365494 0.5166597366333
0.21192836761475 0.97775095701218
0.79371929168701 0.49656808376312
0.78017491102219 0.19556018710136
0.15088713169098 0.9114618897438
0.59973347187042 0.19529362022877
0.53487712144852 0.31236150860786
0.07876718044281 0.242257386446
0.068138279020786 0.67885881662369
0.054010827094316 0.030329030007124
0.30169096589088 0.4191170334816
0.79077053070068 0.68195754289627
0.21512703597546 0.36987090110779
0.98292380571365 0.74816328287125
0.21660974621773 0.79642647504807
0.75291961431503 0.20853811502457
0.14287380874157 0.39266139268875
0.65177839994431 0.4712453186512
0.25922530889511 0.28102457523346
0.65909206867218 0.37301957607269
0.25954186916351 0.012536442838609
0.18302373588085 0.17916701734066
0.26627239584923 0.56678885221481
0.060308203101158 0.58356517553329
0.83043730258942 0.35194501280785
0.55625158548355 0.70866304636002
0.93162846565247 0.20473968982697
0.36473137140274 0.10548105090857
0.97800916433334 0.3904789686203
0.60819655656815 0.75361096858978
0.80631405115128 0.60304039716721
0.8376676440239 0.061216160655022
0.68184924125671 0.96487295627594
0.032736361026764 0.86503124237061
0.29849228262901 0.026997083798051
0.60156601667404 0.43036234378815
0.77869218587875 0.76900458335876
0.38089129328728 0.45108705759048
0.6734693646431 0.59905874729156
0.63601833581924 0.049654312431812
0.96241563558578 0.35389420390129
0.060824654996395 0.77708452939987
0.053694292902946 0.29881715774536
0.77775925397873 0.61296957731247
0.78403997421265 0.12770512700081
0.33784255385399 0.96547269821167
0.41875883936882 0.96300709247589
0.72066640853882 0.67132860422134
0.65172845125198 0.35574343800545
0.33439400792122 0.99584341049194
0.60539776086807 0.28550603985786
0.015760099515319 0.6328946352005
0.83078718185425 0.16045814752579
0.030070804059505 0.70493125915527
0.30748856067657 0.81733447313309
0.071203663945198 0.76297378540039
0.4436651468277 0.52144104242325
0.26160767674446 0.78661388158798
0.076051644980907 0.96665555238724
0.15230320394039 0.18401499092579
0.4699541926384 0.27542689442635
0.7228821516037 0.79191172122955
0.31925031542778 0.99877548217773
0.38150769472122 0.87561017274857
0.74638903141022 0.11629321426153
0.96491461992264 0.1289879232645
0.3023906648159 0.036143273115158
0.73840898275375 0.67449396848679
0.96684712171555 0.59506040811539
0.39586839079857 0.063148684799671
0.14792169630527 0.058350685983896
0.059741772711277 0.27454394102097
0.68234902620316 0.1198917105794
0.28104954957962 0.71684300899506
0.83543521165848 0.90213245153427
0.039383590221405 0.22678050398827
0.30105790495872 0.95609331130981
0.74290710687637 0.069662638008595
0.20166596770287 0.26136609911919
0.22835485637188 0.93936693668365
0.088279880583286 0.22611412405968
0.76103287935257 0.95834237337112
0.94049143791199 0.68707203865051
0.034818824380636 0.67760932445526
0.166463971138 0.90954601764679
0.17216159403324 0.076759681105614
0.60293209552765 0.58741354942322
0.72408163547516 0.56395667791367
0.51520198583603 0.8431236743927
0.67017078399658 0.97593504190445
0.59715116024017 0.22769679129124
0.41859224438667 0.57800084352493
0.97122865915298 0.72072470188141
0.80608081817627 0.064031653106213
0.18845480680466 0.73037070035934
0.097942523658276 0.55647647380829
0.22842149436474 0.09336943179369
0.98805499076843 0.60635566711426
0.49929195642471 0.075027070939541
0.59886711835861 0.19326114654541
0.83780092000961 0.36922115087509
0.48139941692352 0.72535610198975
0.50925445556641 0.098400667309761
0.61524367332458 0.039375260472298
0.20753020048141 0.81358599662781
0.40856310725212 0.40062475204468
0.05507704988122 0.49436900019646
0.69809246063232 0.50298207998276
0.60291546583176 0.54891294240952
0.74913787841797 0.4688962996006
0.83063721656799 0.81395250558853
0.25557684898376 0.84938776493073
0.14640566706657 0.55479383468628
0.33985838294029 0.62404829263687
0.3869554400444 0.46531444787979
0.55300289392471 0.20104123651981
0.81759268045425 0.66796332597733
0.87460225820541 0.41710120439529
0.13219492137432 0.71376091241837
0.71281963586807 0.53753435611725
0.45321115851402 0.58229905366898
0.90438985824585 0.25621825456619
0.33166182041168 0.68174093961716
0.71461892127991 0.69560182094574
0.74713867902756 0.84882134199142
0.83738440275192 0.4067055284977
0.10780508071184 0.34884631633759
0.39513537287712 0.3691211938858
0.25039565563202 0.87569344043732
0.93889212608337 0.99101209640503
0.44021657109261 0.55181175470352
0.44824656844139 0.10911286622286
0.37114536762238 0.92822158336639
0.33136191964149 0.98872971534729
0.1656309813261 0.984514772892
0.42497292160988 0.32374009490013
0.37469387054443 0.12885464727879
0.99438565969467 0.23659308254719
0.97792583703995 0.19797584414482
0.73347771167755 0.27830904722214
0.3834902048111 0.45718449354172
0.76469802856445 0.42847979068756
0.42812162637711 0.60035818815231
0.63906705379486 0.09526863694191
0.37712618708611 0.74994587898254
0.33617660403252 0.11541024595499
0.92438149452209 0.45696792006493
'.
	dots := OrderedCollection new.
	stream skipSeparators.
	[stream atEnd]
		whileFalse: 
			[strm := ReadStream on: (stream upTo: Character cr).
			x := (strm upTo: Character space) asNumber.
			y := (strm upTo: Character cr) asNumber.
			dots add: x @ y.
			stream skipSeparators].
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 200 @ 200.
	translation := 50 @ 50.
	box := ((0 @ 0 extent: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/12/98 02:40'!
example6
	"This is a low precision example."
	"Voronoi2dProcessor example6."

	| dots voronoi triangles gc scale translation box points view |
	dots := OrderedCollection new.
	dots add: 0.1 @ 0.1.
	dots add: 0.1 @ 0.9.
	dots add: 0.9 @ 0.1.
	dots add: 0.9 @ 0.9.
	dots add: 0.5 @ 0.5.
	dots add: 0.1 @ 0.5.
	dots add: 0.9 @ 0.5.
	dots add: 0.5 @ 0.1.
	dots add: 0.5 @ 0.9.
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 200 @ 200.
	translation := 50 @ 50.
	box := ((0 @ 0 extent: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/14/98 02:25'!
example7
	"This is a low precision example."
	"Voronoi2dProcessor example7."

	| dots voronoi triangles gc scale translation box points count view |
	dots := OrderedCollection new.
	count := 0.
	(0 to: 10 by: 1)
		do: [:y | (0 to: 10 by: 1)
				do: 
					[:x | 
					count even ifTrue: [dots add: x @ y / (10 @ 10)].
					count := count + 1]].
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 200 @ 200.
	translation := 50 @ 50.
	box := ((0 @ 0 corner: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/14/98 02:36'!
example8
	"This is a low precision example."
	"Voronoi2dProcessor example8."

	| dots step voronoi x y triangles gc scale translation box points view |
	dots := OrderedCollection new.
	dots add: 0.0 @ 0.0.
	step := 10.
	(0 to: 360 by: step)
		do: 
			[:degrees | 
			x := degrees asFloat degreesToRadians cos.
			y := degrees asFloat degreesToRadians sin.
			dots add: x @ y.
			x := (degrees + (step / 2)) asFloat degreesToRadians cos.
			y := (degrees + (step / 2)) asFloat degreesToRadians sin.
			dots add: x @ y * (0.5 @ 0.5)].
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 100 @ 100.
	translation := 150 @ 150.
	box := ((-1 @ -1 corner: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !

!Voronoi2dProcessor class methodsFor: 'examples' stamp: 'nishis 5/14/98 02:38'!
example9
	"This is a low precision example."
	"Voronoi2dProcessor example9."

	| dots voronoi triangles gc scale translation box points view |
	dots := OrderedCollection new.
	dots add: 0.0 @ 0.0.
	dots add: 0.5 @ 0.0.
	dots add: 0.43301270189222 @ 0.25.
	dots add: 0.86602540378444 @ 0.0.
	dots add: 0.75 @ 0.43301270189222.
	dots add: 1.0 @ 0.0.
	dots add: 0.86602540378444 @ 0.5.
	dots add: 0.25 @ 0.43301270189222.
	dots add: 0.43301270189222 @ 0.75.
	dots add: 0.5 @ 0.86602540378444.
	dots add: 0.0 @ 0.5.
	dots add: 0.0 @ 0.86602540378444.
	dots add: 0.0 @ 1.0.
	dots add: -0.25 @ 0.43301270189222.
	dots add: -0.43301270189222 @ 0.75.
	dots add: -0.5 @ 0.86602540378444.
	dots add: -0.43301270189222 @ 0.25.
	dots add: -0.75 @ 0.43301270189222.
	dots add: -0.86602540378444 @ 0.5.
	dots add: -0.5 @ 0.0.
	dots add: -0.86602540378444 @ 0.0.
	dots add: -1.0 @ 0.0.
	dots add: -0.43301270189222 @ -0.25.
	dots add: -0.75 @ -0.43301270189222.
	dots add: -0.86602540378444 @ -0.5.
	dots add: -0.25 @ -0.43301270189222.
	dots add: -0.43301270189222 @ -0.75.
	dots add: -0.5 @ -0.86602540378444.
	dots add: 0.0 @ -0.5.
	dots add: 0.0 @ -0.86602540378444.
	dots add: 0.0 @ -1.0.
	dots add: 0.25 @ -0.43301270189222.
	dots add: 0.43301270189222 @ -0.75.
	dots add: 0.5 @ -0.86602540378444.
	dots add: 0.43301270189222 @ -0.25.
	dots add: 0.75 @ -0.43301270189222.
	dots add: 0.86602540378444 @ -0.5.
	voronoi := Voronoi2dProcessor dots: dots.
	voronoi trace: true.
	voronoi compute.
	Transcript cr; show: 'area: ' , voronoi area printString.
	triangles := voronoi triangles.
	view := ScheduledControllers activeController view.
	gc := Form extent: view insetDisplayBox extent depth: Display depth.
	gc fillWithColor: Color white.
	gc borderWidth: 2 color: Color black.
	scale := 100 @ 100.
	translation := 150 @ 150.
	box := ((-1 @ -1 corner: 1 @ 1)
				scaleBy: scale)
				translateBy: translation.
	voronoi 
			displayBorderOn: gc 
			box: box 
			lineWidth: 1 
			color: Color red.
	triangles
		do: 
			[:triangle | 
			points := Array
						with: (triangle at: 1)
						with: (triangle at: 2)
						with: (triangle at: 3)
						with: (triangle at: 1).
			points := points collect: [:point | (point scaleBy: scale)
							translateBy: translation].
			voronoi 
					displayPolylineOn: gc 
					points: points 
					lineWidth: 1 
					color: Color black].
	gc displayAt: view insetDisplayBox origin.
	Cursor crossHair showWhile: [Sensor waitClickButton].
	view display.
	^voronoi! !
