'Voronoi (11/18/2001 8:35:30 am)'.!


Voronoi2dObject subclass: #Voronoi2dSide
	instanceVariableNames: 'from to dot1 dot2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Voronoi-2d-Diagram'!

!Voronoi2dSide methodsFor: 'initialize-release'!
initialize
	super initialize.
	from := nil.
	to := nil.
	dot1 := nil.
	dot2 := nil! !


!Voronoi2dSide methodsFor: 'accessing'!
dot1
	^dot1! !

!Voronoi2dSide methodsFor: 'accessing'!
dot1: voronoi2dDot 
	dot1 := voronoi2dDot! !

!Voronoi2dSide methodsFor: 'accessing'!
dot2
	^dot2! !

!Voronoi2dSide methodsFor: 'accessing'!
dot2: voronoi2dDot 
	dot2 := voronoi2dDot! !

!Voronoi2dSide methodsFor: 'accessing'!
from
	^from! !

!Voronoi2dSide methodsFor: 'accessing'!
from: voronoi2dPoint 
	from := voronoi2dPoint! !

!Voronoi2dSide methodsFor: 'accessing'!
to
	^to! !

!Voronoi2dSide methodsFor: 'accessing'!
to: voronoi2dPoint 
	to := voronoi2dPoint! !


!Voronoi2dSide methodsFor: 'testing'!
isInvalid
	^self isValid not! !

!Voronoi2dSide methodsFor: 'testing'!
isValid
	| boolean |
	boolean := true.
	from isNil
		ifTrue: [boolean := false]
		ifFalse: [from isValid ifFalse: [boolean := false]].
	to isNil
		ifTrue: [boolean := false]
		ifFalse: [to isValid ifFalse: [boolean := false]].
	dot1 isNil ifTrue: [boolean := false].
	dot2 isNil ifTrue: [boolean := false].
	^boolean! !

!Voronoi2dSide methodsFor: 'testing'!
isValid2
	| boolean |
	boolean := true.
	from isNil ifTrue: [boolean := false].
	to isNil ifTrue: [boolean := false].
	dot1 isNil ifTrue: [boolean := false].
	dot2 isNil ifTrue: [boolean := false].
	^boolean! !


!Voronoi2dSide methodsFor: 'functions'!
intersectingPointWithSide: voronoi2dSide 
	| x1 y1 x2 y2 x3 y3 x4 y4 x21 y21 x43 y43 x31 y31 det detinv s t x y |
	x1 := self from x.
	y1 := self from y.
	x2 := self to x.
	y2 := self to y.
	x3 := voronoi2dSide from x.
	y3 := voronoi2dSide from y.
	x4 := voronoi2dSide to x.
	y4 := voronoi2dSide to y.
	x21 := x2 - x1.
	y21 := y2 - y1.
	x43 := x4 - x3.
	y43 := y4 - y3.
	x31 := x3 - x1.
	y31 := y3 - y1.
	det := x43 * y21 - (y43 * x21).
	det abs < self accuracy ifTrue: [^nil].
	detinv := 1.0 / det.
	s := x43 * y31 - (y43 * x31) * detinv.
	t := x21 * y31 - (y21 * x31) * detinv.
	(s < 0.0 or: [1.0 < s or: [t < 0.0 or: [1.0 < t]]])
		ifTrue: [^nil].
	x := x1 + (x21 * s).
	y := y1 + (y21 * s).
	^Voronoi2dPoint x: x y: y! !


!Voronoi2dSide methodsFor: 'printing'!
printOn: aStream 
	aStream nextPutAll: 'side ( '.
	from printOn: aStream.
	aStream nextPutAll: ' , '.
	to printOn: aStream.
	self
		debug: 
			[aStream nextPutAll: ' , '.
			dot1 isNil
				ifTrue: [aStream nextPutAll: 'invalid dot1']
				ifFalse: [dot1 printOn: aStream].
			aStream nextPutAll: ' , '.
			dot2 isNil
				ifTrue: [aStream nextPutAll: 'invalid dot2']
				ifFalse: [dot2 printOn: aStream]].
	aStream nextPutAll: ' )'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Voronoi2dSide class
	instanceVariableNames: ''!

!Voronoi2dSide class methodsFor: 'copyright'!
copyright
	^'Copyright 1991-1998 AOKI Atsushi'! !

!Voronoi2dSide class methodsFor: 'copyright'!
system
	^'Voronoi and Delaunay Diagram'! !

!Voronoi2dSide class methodsFor: 'copyright'!
version
	^'006'! !


!Voronoi2dSide class methodsFor: 'instance creation'!
from: voronoi2dPoint1 to: voronoi2dPoint2 
	^(super new) from: voronoi2dPoint1; to: voronoi2dPoint2; yourself! !


!Voronoi2dSide class methodsFor: 'examples'!
example1
	"Voronoi2dSide example1."

	| side |
	side := Voronoi2dSide from: (Voronoi2dPoint x: 0 y: 0)
				to: (Voronoi2dPoint x: 1 y: 1).
	^side! !

!Voronoi2dSide class methodsFor: 'examples'!
example2
	"Voronoi2dSide example2."

	| side1 side2 point |
	side1 := Voronoi2dSide from: (Voronoi2dPoint x: 0 y: 0)
				to: (Voronoi2dPoint x: 1 y: 1).
	side2 := Voronoi2dSide from: (Voronoi2dPoint x: 1 y: 0)
				to: (Voronoi2dPoint x: 0 y: 1).
	point := side1 intersectingPointWithSide: side2.
	^point! !
