'Voronoi (11/18/2001 8:35:30 am)'.!


Voronoi2dObject subclass: #Voronoi2dLine
	instanceVariableNames: 'a b c '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Voronoi-2d-Diagram'!

!Voronoi2dLine methodsFor: 'initialize-release'!
initialize
	super initialize.
	a := nil.
	b := nil.
	c := nil! !


!Voronoi2dLine methodsFor: 'accessing'!
a
	^a! !

!Voronoi2dLine methodsFor: 'accessing'!
a: aNumber 
	a := aNumber asFloat! !

!Voronoi2dLine methodsFor: 'accessing'!
b
	^b! !

!Voronoi2dLine methodsFor: 'accessing'!
b: aNumber 
	b := aNumber asFloat! !

!Voronoi2dLine methodsFor: 'accessing'!
c
	^c! !

!Voronoi2dLine methodsFor: 'accessing'!
c: aNumber 
	c := aNumber asFloat! !

!Voronoi2dLine methodsFor: 'accessing'!
pointAtX: x 
	| y |
	y := c negated - (a * x) / b.
	^Voronoi2dPoint x: x y: y! !

!Voronoi2dLine methodsFor: 'accessing'!
pointAtY: y 
	| x |
	x := c negated - (b * y) / a.
	^Voronoi2dPoint x: x y: y! !


!Voronoi2dLine methodsFor: 'testing'!
isHorizontal
	^a abs < self accuracy! !

!Voronoi2dLine methodsFor: 'testing'!
isVertical
	^b abs < self accuracy! !


!Voronoi2dLine methodsFor: 'functions'!
intersectingPointWithLine: voronoi2dLine 
	| a1 b1 c1 a2 b2 c2 det dinv x y |
	a1 := self a.
	b1 := self b.
	c1 := self c.
	a2 := voronoi2dLine a.
	b2 := voronoi2dLine b.
	c2 := voronoi2dLine c.
	det := a1 * b2 - (a2 * b1).
	det abs < self accuracy ifTrue: [^self error: 'two lines are parallel.'].
	dinv := 1.0 / det.
	x := b1 * c2 - (b2 * c1) * dinv.
	y := a2 * c1 - (a1 * c2) * dinv.
	^Voronoi2dPoint x: x y: y! !


!Voronoi2dLine methodsFor: 'printing'!
printOn: aStream 
	aStream nextPutAll: 'line ( '.
	a printOn: aStream.
	aStream nextPutAll: ' , '.
	b printOn: aStream.
	aStream nextPutAll: ' , '.
	c printOn: aStream.
	aStream nextPutAll: ' )'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Voronoi2dLine class
	instanceVariableNames: ''!

!Voronoi2dLine class methodsFor: 'copyright'!
copyright
	^'Copyright 1991-1998 AOKI Atsushi'! !

!Voronoi2dLine class methodsFor: 'copyright'!
system
	^'Voronoi and Delaunay Diagram'! !

!Voronoi2dLine class methodsFor: 'copyright'!
version
	^'006'! !


!Voronoi2dLine class methodsFor: 'instance creation'!
a: aNumber b: bNumber c: cNumber 
	^(super new) a: aNumber; b: bNumber; c: cNumber; yourself! !

!Voronoi2dLine class methodsFor: 'instance creation'!
from: voronoi2dDot1 to: voronoi2dDot2 
	| x1 y1 x2 y2 a b c |
	x1 := voronoi2dDot1 x.
	y1 := voronoi2dDot1 y.
	x2 := voronoi2dDot2 x.
	y2 := voronoi2dDot2 y.
	a := y1 - y2.
	b := x2 - x1.
	c := x1 * y2 - (x2 * y1).
	(a abs < self accuracy and: [b abs < self accuracy])
		ifTrue: [^self error: 'two points are the same.'].
	^self
		a: a
		b: b
		c: c! !


!Voronoi2dLine class methodsFor: 'examples'!
example1
	"Voronoi2dLine example1."

	| line |
	line := Voronoi2dLine from: (Voronoi2dDot x: 0 y: 0)
				to: (Voronoi2dDot x: 1 y: 1).
	^line! !

!Voronoi2dLine class methodsFor: 'examples'!
example2
	"Voronoi2dLine example2."

	| line1 line2 dot |
	line1 := Voronoi2dLine from: (Voronoi2dDot x: 0 y: 0)
				to: (Voronoi2dDot x: 1 y: 1).
	line2 := Voronoi2dLine from: (Voronoi2dDot x: 1 y: 0)
				to: (Voronoi2dDot x: 0 y: 1).
	dot := line1 intersectingPointWithLine: line2.
	^dot! !
